#include <NTPClient.h>

#include <Arduino.h>

#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266HTTPUpdateServer.h>

// sonoff basic
// gpio 2 ds18b20 D4
// gpio 0 button D3
// gpio 12 relais D6
// gpio 13 led1 D7

// sonoff 4ch r2
// button 00 09 10 14
// Relay  12 05 04 15
// led    13

//#define has_ntp
#if defined has_ntp
  #define ntptimeoffset 7200  // 2 h in sec
  #include <NTPClient.h>
  #include <WiFiUdp.h>
  
  WiFiUDP ntpUDP;
  NTPClient timeClient(ntpUDP);
#else
  #define timeinterval 20000 // 1 minute in ms
#endif

#define num_sensors_max 6
int num_sensors = 0;

#include "OneWire.h"
#define onewirepin 2
OneWire ds(onewirepin); // This is GPIO, not pin
byte onewireaddr[num_sensors_max][8];

ESP8266WebServer server(80);
ESP8266HTTPUpdateServer httpUpdater;

const char *ssid = "FSM";
const char *password = "0101010101";

void setup() 
{
  Serial.begin(115200);  
  Serial.println("Start");
  Serial.flush();
  delay(500);
  
  Serial.println("OneWire search");
  ds.reset_search();
  for(int k=0;k<num_sensors_max;k++)
  {
    byte localaddr[8];
    if (ds.search(localaddr))
    { 
      num_sensors++;
      for (int i=0;i<8;i++) onewireaddr[k][i] = localaddr[i];
      Serial.println("Found DS18B20");
      delay(250);
    }
    else break;
  }

  WiFi.mode(WIFI_STA);

  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) 
  {
    delay(500);
    Serial.print(".");
  }
  Serial.println(" connected to WiFi");
  Serial.println(WiFi.localIP());
  WiFi.setSleepMode(WIFI_NONE_SLEEP);

  httpUpdater.setup(&server,"/up"); 

  server.on("/", drawGraph);
  server.on("/cmd", handleCmd);
  server.onNotFound(handleNotFound);
  server.begin();
  Serial.println("HTTP server started");

#if defined has_ntp
  timeClient.begin();
  timeClient.setTimeOffset(ntptimeoffset); // in sec
  timeClient.setUpdateInterval(43200000UL); // in ms 12h
  timeClient.update();
  if (timeClient.isTimeSet())
  {
    Serial.print("NTP time");
    Serial.println(timeClient.getFormattedTime());
  }
#endif
  clearwp();
  switchinit();
}

void inittemp(byte addr[8])
{
  ds.reset();
  ds.select(addr);
  ds.write(0x44);         
}

float gettemp(byte addr[8])
{
  byte data_0,data_1;
  ds.reset();
  ds.select(addr);
  ds.write(0xBE);
  data_0 = ds.read();
  data_1 = ds.read();
  
  int16_t raw = (data_1 << 8) | data_0;
  return (float)raw / 16.0;
}

unsigned long nextTime;
int actMinute;
int actHour = 0;

float celsius3[num_sensors_max];

#define kessel    0 // red
#define vorlauf   1 // green
#define rücklauf  2 // magenta
#define abgas     3 // orange
#define vorlauf2  4 // lime
#define rücklauf2 5 // yellow

bool IsActive = false;
bool IsFloor = false; 

int16_t HIlimit = 55; 
int16_t LOlimit = 45; 

int16_t HIpumpe1 = 35; // Switch on above
int16_t LOpumpe1 = 30; // Switch off below
int16_t ONpumpe2 = 30; // Switch on above
int16_t OFpumpe2 = 28; // Switch off below

void loop() 
{
  //Serial.print(num_sensors); Serial.print("  ");  
  for (int k=0;k<num_sensors;k++)
  {
    inittemp(onewireaddr[k]);
  }
  delay(1000);     
  for (int k=0;k<num_sensors;k++)
  {
    float celsius = gettemp(onewireaddr[k]);
    Serial.print(celsius); Serial.print("  ");  
    celsius3[k] = celsius;
  }
  
  if (WiFi.status() == WL_CONNECTED) 
  {
    // 720 points, every min -> 12h.
#if defined has_ntp
    actHour = timeClient.getHours();
#endif
    storewp(celsius3, actHour);
  } 

  // Steuerung
  if (IsActive)
  {
    if      (celsius3[kessel]  >= HIlimit)  switchBurner(false); // upper limit, off
    else if (celsius3[vorlauf] >= HIlimit)  switchBurner(false); // upper limit, off
    else if (celsius3[kessel]  <= LOlimit)  switchBurner(true);  // on burner
  }
  else switchBurner(false);
 
  // umwälzpumpe 1
  if      (celsius3[kessel] >= HIpumpe1) switchPumpe1(true);  
  else if (celsius3[kessel] <= LOpumpe1) switchPumpe1(false); 

  // umwälzpumpe 2
  if (IsFloor) 
  {
    if      (celsius3[rücklauf] >= ONpumpe2) switchPumpe2(true); 
    else if (celsius3[rücklauf] <= OFpumpe2) switchPumpe2(false); 
  }
  else switchPumpe2(false);
     
  Serial.println();
 
#if defined has_ntp
  while (timeClient.getMinutes() == actMinute) 
  {
    delay(20);
    server.handleClient();
  }
  actMinute = timeClient.getMinutes();
#else
  while (millis() <= nextTime) 
  {
    delay(20);
    server.handleClient();  
  }
  nextTime = timeinterval + millis();  
#endif
}
